/***************************************************************************
 *   Copyright (C) 2008 by Manuel Schrape                                  *
 *   manuel.schrape@gmx.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>

#include <QLineEdit>
#include <QString>
#include <QTimer>
#include <QIcon>
#include <QToolButton>
#include <QSpinBox>
#include <QMessageBox>

#include "mktool.h"
#include "../global.h"

MKTool::MKTool()
{
    setupUi(this);

    pb_Decode->hide();
    le_In->hide();
    le_Out->hide();

    read_Settings();

    setWindowTitle(QA_NAME + " v" + QA_VERSION);

    timer = new QTimer(this);

    serialPort=serialPort = new ManageSerialPort;
    connect(serialPort, SIGNAL(newDataReceived(const QByteArray &)), this, SLOT(slot_newDataReceived(const QByteArray &)));

//    connect(pb_Decode,  SIGNAL(clicked()), this, SLOT(Decode64()));
    connect(pb_Open,  SIGNAL(clicked()), this, SLOT(slot_OpenPort()));
    connect(pb_Quit,  SIGNAL(clicked()), this, SLOT(slot_Quit()));
    connect(pb_Read,  SIGNAL(clicked()), this, SLOT(slot_GetParameter()));
    connect(pb_Write, SIGNAL(clicked()), this, SLOT(slot_SetParameter()));
    connect(pb_Send,  SIGNAL(clicked()), this, SLOT(slot_Send()));

    connect(timer,    SIGNAL(timeout()),       SLOT(slot_Timer()));

    connect(tb_9_6,  SIGNAL(clicked()), this, SLOT(slot_tbUp()));
    connect(tb_9_7,  SIGNAL(clicked()), this, SLOT(slot_tbDown()));
    connect(tb_9_8,  SIGNAL(clicked()), this, SLOT(slot_tbLeft()));
    connect(tb_9_9,  SIGNAL(clicked()), this, SLOT(slot_tbRight()));

    connect(J16_0,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_1,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_2,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_3,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_4,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_5,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_6,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J16_7,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));

    connect(J17_0,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_1,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_2,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_3,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_4,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_5,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_6,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));
    connect(J17_7,  SIGNAL(clicked()), this, SLOT(slot_LEDtoValue()));

    connect(sb_11_1, SIGNAL(valueChanged(int)), this, SLOT(slot_ValuetoLED16(int)));
    connect(sb_11_3, SIGNAL(valueChanged(int)), this, SLOT(slot_ValuetoLED17(int)));
}

void MKTool::slot_LEDtoValue()
{
    QToolButton *ToolButton = (QToolButton*)sender();
    if (ToolButton->text() == QString("0"))
    {
        set_LED(ToolButton, true);
        sb_11_1->setValue(sb_11_1->value() + ToolButton->toolTip().toInt());
    }
    else if (ToolButton->text() == QString("1"))
    {
        set_LED(ToolButton);
        sb_11_1->setValue(sb_11_1->value() - ToolButton->toolTip().toInt());
    }
    else if (ToolButton->text() == QString("00"))
    {
        set_LED(ToolButton, true);
        sb_11_3->setValue(sb_11_3->value() + ToolButton->toolTip().toInt());
    }
    else if (ToolButton->text() == QString("11"))
    {
        set_LED(ToolButton);
        sb_11_3->setValue(sb_11_3->value() - ToolButton->toolTip().toInt());
    }
}

void MKTool::slot_ValuetoLED16(int Wert)
{
//    QSpinBox *SpinBox = (QSpinBox*)sender();

//    int Wert = SpinBox->value();

    set_LED(J16_0);
    set_LED(J16_1);
    set_LED(J16_2);
    set_LED(J16_3);
    set_LED(J16_4);
    set_LED(J16_5);
    set_LED(J16_6);
    set_LED(J16_7);

    for (int a = 0; a < 8; a++)
    {
        if (Wert > 127)
        {
            set_LED(J16_0, true);
            Wert = Wert - 128;
        }
        if (Wert > 63)
        {
            set_LED(J16_1, true);
            Wert = Wert - 64;
        }
        if (Wert > 31)
        {
            set_LED(J16_2, true);
            Wert = Wert - 32;
        }
        if (Wert > 15)
        {
            set_LED(J16_3, true);
            Wert = Wert - 16;
        }
        if (Wert > 7)
        {
            set_LED(J16_4, true);
            Wert = Wert - 8;
        }
        if (Wert > 3)
        {
            set_LED(J16_5, true);
            Wert = Wert - 4;
        }
        if (Wert > 1)
        {
            set_LED(J16_6, true);
            Wert = Wert - 2;
        }
        if (Wert > 0)
        {
            set_LED(J16_7, true);
            Wert = Wert - 1;
        }
    }
}

void MKTool::slot_ValuetoLED17(int Wert)
{
//    QSpinBox *SpinBox = (QSpinBox*)sender();

//    int Wert = SpinBox->value();

    set_LED(J17_0);
    set_LED(J17_1);
    set_LED(J17_2);
    set_LED(J17_3);
    set_LED(J17_4);
    set_LED(J17_5);
    set_LED(J17_6);
    set_LED(J17_7);

    for (int a = 0; a < 8; a++)
    {
        if (Wert > 127)
        {
            set_LED(J17_0, true);
            Wert = Wert - 128;
        }
        if (Wert > 63)
        {
            set_LED(J17_1, true);
            Wert = Wert - 64;
        }
        if (Wert > 31)
        {
            set_LED(J17_2, true);
            Wert = Wert - 32;
        }
        if (Wert > 15)
        {
            set_LED(J17_3, true);
            Wert = Wert - 16;
        }
        if (Wert > 7)
        {
            set_LED(J17_4, true);
            Wert = Wert - 8;
        }
        if (Wert > 3)
        {
            set_LED(J17_5, true);
            Wert = Wert - 4;
        }
        if (Wert > 1)
        {
            set_LED(J17_6, true);
            Wert = Wert - 2;
        }
        if (Wert > 0)
        {
            set_LED(J17_7, true);
            Wert = Wert - 1;
        }
    }
}

void MKTool::set_LED(QToolButton *ToolButton, bool On)
{
    if (ToolButton->text() == QString("0") && On)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/LED/Images/ledred.png")), QIcon::Normal, QIcon::Off);
        ToolButton->setIcon(Icon);
        ToolButton->setText("1");
    }
    else if (ToolButton->text() == QString("1") && !On)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/LED/Images/ledoff.png")), QIcon::Normal, QIcon::Off);
        ToolButton->setIcon(Icon);
        ToolButton->setText("0");
    }
    else if (ToolButton->text() == QString("00") && On)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/LED/Images/ledred.png")), QIcon::Normal, QIcon::Off);
        ToolButton->setIcon(Icon);
        ToolButton->setText("11");
    }
    else if (ToolButton->text() == QString("11") && !On)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/LED/Images/ledoff.png")), QIcon::Normal, QIcon::Off);
        ToolButton->setIcon(Icon);
        ToolButton->setText("00");
    }
}

void MKTool::slot_tbUp()
{
    if (tb_9_6->text() == QString("0"))
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-up-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_6->setIcon(Icon);
        tb_9_6->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-up.png")), QIcon::Normal, QIcon::Off);
        tb_9_6->setIcon(Icon);
        tb_9_6->setText("0");
    }
}

void MKTool::slot_tbDown()
{
    if (tb_9_7->text() == QString("0"))
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-down-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_7->setIcon(Icon);
        tb_9_7->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-down.png")), QIcon::Normal, QIcon::Off);
        tb_9_7->setIcon(Icon);
        tb_9_7->setText("0");
    }
}

void MKTool::slot_tbLeft()
{
    if (tb_9_8->text() == QString("0"))
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-left-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_8->setIcon(Icon);
        tb_9_8->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-left.png")), QIcon::Normal, QIcon::Off);
        tb_9_8->setIcon(Icon);
        tb_9_8->setText("0");
    }
}

void MKTool::slot_tbRight()
{
    if (tb_9_9->text() == QString("0"))
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-right-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_9->setIcon(Icon);
        tb_9_9->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-right.png")), QIcon::Normal, QIcon::Off);
        tb_9_9->setIcon(Icon);
        tb_9_9->setText("0");
    }
}

void MKTool::read_Settings()
{
    QSettings Setting("KeyOz-Net", "QMK-Groundstation");

    Setting.beginGroup("Port");
        le_Port->setText(Setting.value("TTY", QString("/dev/ttyUSB0")).toString());
    Setting.endGroup();

    Setting.beginGroup("GUI");
        resize(Setting.value("Size", QSize(700, 300)).toSize());
        move(Setting.value("Point",QPoint(1,1)).toPoint());
        if (Setting.value("IsMax",false).toBool())
            showMaximized();
    Setting.endGroup();

    Setting.beginGroup("AnalogWerte");
        lb_A_0->setText(Setting.value("Analog_0", QString("Integral Nick")).toString());
        lb_A_1->setText(Setting.value("Analog_1", QString("Integral Roll")).toString());
        lb_A_2->setText(Setting.value("Analog_2", QString("ACC Nick")).toString());
        lb_A_3->setText(Setting.value("Analog_3", QString("ACC Roll")).toString());
        lb_A_4->setText(Setting.value("Analog_4", QString("Gyro Gier")).toString());
        lb_A_5->setText(Setting.value("Analog_5", QString("Hoehen-Wert")).toString());
        lb_A_6->setText(Setting.value("Analog_6", QString("ACC Z")).toString());
        lb_A_7->setText(Setting.value("Analog_7", QString("GAS")).toString());
        lb_A_8->setText(Setting.value("Analog_8", QString("Kompass-Value")).toString());
        lb_A_9->setText(Setting.value("Analog_9", QString("Spannung")).toString());
        lb_A_10->setText(Setting.value("Analog_10", QString("Empfang")).toString());
        lb_A_11->setText(Setting.value("Analog_11", QString("Ersatzkompass")).toString());
        lb_A_12->setText(Setting.value("Analog_12", QString("Motor Vorne")).toString());
        lb_A_13->setText(Setting.value("Analog_13", QString("Motor Hinten")).toString());
        lb_A_14->setText(Setting.value("Analog_14", QString("Motor Links")).toString());
        lb_A_15->setText(Setting.value("Analog_15", QString("Motor Rechts")).toString());
        lb_A_17->setText(Setting.value("Analog_17", QString("Distance")).toString());
        lb_A_18->setText(Setting.value("Analog_18", QString("OSD-Bar")).toString());
        lb_A_19->setText(Setting.value("Analog_19", QString("MK3Mag")).toString());
        lb_A_20->setText(Setting.value("Analog_20", QString("Servo")).toString());
        lb_A_21->setText(Setting.value("Analog_21", QString("Nick")).toString());
        lb_A_22->setText(Setting.value("Analog_22", QString("Roll")).toString());
        lb_A_30->setText(Setting.value("Analog_30", QString("GPS Nick")).toString());
        lb_A_31->setText(Setting.value("Analog_31", QString("GPS Roll")).toString());
    Setting.endGroup();
}

void MKTool::write_Settings()
{
    QSettings Setting("KeyOz-Net", "QMK-Groundstation");

    Setting.beginGroup("Port");
        Setting.setValue("TTY", le_Port->text());
    Setting.endGroup();

    Setting.beginGroup("GUI");
        Setting.setValue("IsMax", isMaximized());
        Setting.setValue("Size",  size());
        Setting.setValue("Point", pos());
    Setting.endGroup();

    Setting.beginGroup("AnalogWerte");
        Setting.setValue("Analog_0", lb_A_0->text());
        Setting.setValue("Analog_1", lb_A_1->text());
        Setting.setValue("Analog_2", lb_A_2->text());
        Setting.setValue("Analog_3", lb_A_3->text());
        Setting.setValue("Analog_4", lb_A_4->text());
        Setting.setValue("Analog_5", lb_A_5->text());
        Setting.setValue("Analog_6", lb_A_6->text());
        Setting.setValue("Analog_7", lb_A_7->text());
        Setting.setValue("Analog_8", lb_A_8->text());
        Setting.setValue("Analog_9", lb_A_9->text());
        Setting.setValue("Analog_10", lb_A_10->text());
        Setting.setValue("Analog_11", lb_A_11->text());
        Setting.setValue("Analog_12", lb_A_12->text());
        Setting.setValue("Analog_13", lb_A_13->text());
        Setting.setValue("Analog_14", lb_A_14->text());
        Setting.setValue("Analog_15", lb_A_15->text());
        Setting.setValue("Analog_16", lb_A_16->text());
        Setting.setValue("Analog_17", lb_A_17->text());
        Setting.setValue("Analog_18", lb_A_18->text());
        Setting.setValue("Analog_19", lb_A_19->text());
        Setting.setValue("Analog_20", lb_A_20->text());
        Setting.setValue("Analog_21", lb_A_21->text());
        Setting.setValue("Analog_22", lb_A_22->text());
        Setting.setValue("Analog_23", lb_A_23->text());
        Setting.setValue("Analog_24", lb_A_24->text());
        Setting.setValue("Analog_25", lb_A_25->text());
        Setting.setValue("Analog_26", lb_A_26->text());
        Setting.setValue("Analog_27", lb_A_27->text());
        Setting.setValue("Analog_28", lb_A_28->text());
        Setting.setValue("Analog_29", lb_A_29->text());
        Setting.setValue("Analog_30", lb_A_30->text());
        Setting.setValue("Analog_31", lb_A_31->text());
    Setting.endGroup();
}

void MKTool::Decode64(QString Data, bool Long)
{
    QString OUT = "";

    char *RxdBuffer;
    unsigned char a,b,c,d;
    unsigned char ptr = 0;
    unsigned char x,y,z;
    int ptrOut[150];

    int len = Data.length();
    int ptrIn = 0;
    int max = len;
    int DecLen = 0;

    Data.remove(0,3);

    RxdBuffer = Data.toLatin1().data();

    while(len != 0)
    {
        a = RxdBuffer[ptrIn++] - '=';
        b = RxdBuffer[ptrIn++] - '=';
        c = RxdBuffer[ptrIn++] - '=';
        d = RxdBuffer[ptrIn++] - '=';

        if(ptrIn > max - 2) break; // nicht mehr Daten verarbeiten, als empfangen wurden

        x = (a << 2) | (b >> 4);
        y = ((b & 0x0f) << 4) | (c >> 2);
        z = ((c & 0x03) << 6) | d;

        if(len--) ptrOut[ptr++] = x; else break;
        if(len--) ptrOut[ptr++] = y; else break;
        if(len--) ptrOut[ptr++] = z; else break;
    }

    for (int a=0; a<ptr; a++)
    {
        if (Long == false)
        {
            int b1, b2, b3;

            b1 = ptrOut[a++];
            b2 = ptrOut[a];

            b3 = (b2 << 8) | b1;

            if (b3 > 32767)
                b3 = b3 - 65536;

            DecodeData[DecLen] = b3;
            DecLen++;
                OUT = OUT + QString(",%1").arg(b3);
        }
        else
        {
            DecodeData[DecLen] = ptrOut[a];
            DecLen++;
            OUT = OUT + QString(",%1").arg(ptrOut[a]);
        }
    }
    le_Out->setText(OUT);
}

QString MKTool::Encode64(char Data[150],unsigned int Length)
{
    unsigned int pt = 0;
    unsigned char a,b,c;
    unsigned char ptr = 0;

    char TX_Buff[150];

    while(Length > 0)
    {
        if(Length) { a = Data[ptr++]; Length--;} else a = 0;
        if(Length) { b = Data[ptr++]; Length--;} else b = 0;
        if(Length) { c = Data[ptr++]; Length--;} else c = 0;

        TX_Buff[pt++] = '=' + (a >> 2);
        TX_Buff[pt++] = '=' + (((a & 0x03) << 4) | ((b & 0xf0) >> 4));
        TX_Buff[pt++] = '=' + (((b & 0x0f) << 2) | ((c & 0xc0) >> 6));
        TX_Buff[pt++] = '=' + ( c & 0x3f);
    }
    TX_Buff[pt] = 0;

    return QString(TX_Buff);
}

void MKTool::send_Data(char CMD, char Data[150],unsigned int Length)
{

    QString TX_Data = Encode64(Data, Length);

    TX_Data = QString("#0") + QString(CMD) + TX_Data;

    qDebug(TX_Data.toLatin1().data());

    TX_Data = add_CRC(TX_Data) + '\r';

    qDebug(TX_Data.toLatin1().data());

    QByteArray Temp(TX_Data.toUtf8());
    LastSend = TX_Data;

    timer->start(1000);

    serialPort->sendData(Temp);
}

void MKTool::show_DebugData()
{
    le_A_0->setText(QString("%1").arg(AnalogData[0]));
    le_A_1->setText(QString("%1").arg(AnalogData[1]));
    le_A_2->setText(QString("%1").arg(AnalogData[2]));
    le_A_3->setText(QString("%1").arg(AnalogData[3]));
    le_A_4->setText(QString("%1").arg(AnalogData[4]));
    le_A_5->setText(QString("%1").arg(AnalogData[5]));
    le_A_6->setText(QString("%1").arg(AnalogData[6]));
    le_A_7->setText(QString("%1").arg(AnalogData[7]));
    le_A_8->setText(QString("%1").arg(AnalogData[8]));
    le_A_9->setText(QString("%1").arg(AnalogData[9]));
    le_A_10->setText(QString("%1").arg(AnalogData[10]));
    le_A_11->setText(QString("%1").arg(AnalogData[11]));
    le_A_12->setText(QString("%1").arg(AnalogData[12]));
    le_A_13->setText(QString("%1").arg(AnalogData[13]));
    le_A_14->setText(QString("%1").arg(AnalogData[14]));
    le_A_15->setText(QString("%1").arg(AnalogData[15]));
    le_A_16->setText(QString("%1").arg(AnalogData[16]));
    le_A_17->setText(QString("%1").arg(AnalogData[17]));
    le_A_18->setText(QString("%1").arg(AnalogData[18]));
    le_A_19->setText(QString("%1").arg(AnalogData[19]));
    le_A_20->setText(QString("%1").arg(AnalogData[20]));
    le_A_21->setText(QString("%1").arg(AnalogData[21]));
    le_A_22->setText(QString("%1").arg(AnalogData[22]));
    le_A_23->setText(QString("%1").arg(AnalogData[23]));
    le_A_24->setText(QString("%1").arg(AnalogData[24]));
    le_A_25->setText(QString("%1").arg(AnalogData[25]));
    le_A_26->setText(QString("%1").arg(AnalogData[26]));
    le_A_27->setText(QString("%1").arg(AnalogData[27]));
    le_A_28->setText(QString("%1").arg(AnalogData[28]));
    le_A_29->setText(QString("%1").arg(AnalogData[29]));
    le_A_30->setText(QString("%1").arg(AnalogData[30]));
    le_A_31->setText(QString("%1").arg(AnalogData[31]));
}

QComboBox *MKTool::setCombo(QComboBox *Combo, int Set, int Wert)
{
    if (ParameterSet[Set][Wert] <= 250)
    {
        Combo->setItemText(4, QString("%1").arg(ParameterSet[Set][Wert]));
        Combo->setCurrentIndex(4);
    }
    else
    {
        Combo->setCurrentIndex(ParameterSet[Set][Wert] - 251);
    }
    return Combo;
}

int MKTool::get_Value(QComboBox *Combo)
{
    if (Combo->currentText() == QString("Poti 1"))
        return 251;
    if (Combo->currentText() == QString("Poti 2"))
        return 252;
    if (Combo->currentText() == QString("Poti 3"))
        return 253;
    if (Combo->currentText() == QString("Poti 4"))
        return 254;
    return Combo->currentText().toInt();
}

void MKTool::store_ParameterSet(int Set)
{
    char *Name = le_SetName->text().toLatin1().data();

    int a;

    for (a = 0; a < le_SetName->text().length(); a++)
    {
        ParameterSet[Set][P_NAME+a] = Name[a];
    }

    while(a < 12)
    {
        ParameterSet[Set][P_NAME+a] = 0;
        a++;
    }

    // Seite 1
    ParameterSet[Set][P_GLOBAL_CONF] = 0;

    if (cb_1_1->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x01;
    if (cb_1_2->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x02;
    if (cb_1_3->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x04;
    if (cb_1_4->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x08;
    if (cb_1_5->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x10;
    if (cb_1_6->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x20;
    if (cb_1_7->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x40;
    if (cb_1_8->isChecked())
        ParameterSet[Set][P_GLOBAL_CONF] = ParameterSet[Set][P_GLOBAL_CONF] | 0x80;

    // Seite 2
    ParameterSet[Set][P_KANAL_NICK]  = sb_2_1->value();
    ParameterSet[Set][P_KANAL_ROLL]  = sb_2_2->value();
    ParameterSet[Set][P_KANAL_GAS]   = sb_2_3->value();
    ParameterSet[Set][P_KANAL_GIER]  = sb_2_4->value();
    ParameterSet[Set][P_KANAL_POTI1] = sb_2_5->value();
    ParameterSet[Set][P_KANAL_POTI2] = sb_2_6->value();
    ParameterSet[Set][P_KANAL_POTI3] = sb_2_7->value();
    ParameterSet[Set][P_KANAL_POTI4] = sb_2_8->value();

    // Seite 3
    ParameterSet[Set][P_STICK_P]  = sb_3_1->value();
    ParameterSet[Set][P_STICK_D]  = sb_3_2->value();
    ParameterSet[Set][P_GIER_P]   = get_Value(cb_3_3);
    ParameterSet[Set][P_EXTERNAL] = get_Value(cb_3_4);

    // Seite 4
    ParameterSet[Set][P_MAXHOEHE]   = get_Value(cb_4_1);
    ParameterSet[Set][P_MIN_GAS]    = sb_4_2->value();
    ParameterSet[Set][P_HOEHE_P]    = get_Value(cb_4_3);
    ParameterSet[Set][P_DRUCK_D]    = get_Value(cb_4_4);
    ParameterSet[Set][P_HOEHE_ACC]  = get_Value(cb_4_5);
    ParameterSet[Set][P_HOEHE_GAIN] = sb_4_6->value();

    // Seite 5
    ParameterSet[Set][P_GYRO_P]          = get_Value(cb_5_1);
    ParameterSet[Set][P_GYRO_I]          = get_Value(cb_5_2);
    ParameterSet[Set][P_DYNAMIC_STAB]    = get_Value(cb_5_3);
    ParameterSet[Set][P_GYRO_ACC_FAKTOR] = sb_5_4->value();
    ParameterSet[Set][P_GYRO_ACC_ABGL]   = sb_5_5->value();
    ParameterSet[Set][P_FAKTOR_I]        = get_Value(cb_5_6);
    ParameterSet[Set][P_DRIFT_KOMP]      = sb_5_7->value();

    // Seite 6
    ParameterSet[Set][P_SERVO_NICK_CONT]  = get_Value(cb_6_1);
    ParameterSet[Set][P_SERVO_NICK_COMP]  = sb_6_2->value();
    ParameterSet[Set][P_SERVO_NICK_MIN]   = sb_6_3->value();
    ParameterSet[Set][P_SERVO_NICK_MAX]   = sb_6_4->value();
    ParameterSet[Set][P_SERVO_NICK_REFR]  = sb_6_5->value();
    ParameterSet[Set][P_SERVO_NICK_COMPI] = cb_6_6->isChecked();

    // Seite 7
    ParameterSet[Set][P_GAS_MAX]         = sb_7_1->value();
    ParameterSet[Set][P_GAS_MIN]         = sb_7_2->value();
    ParameterSet[Set][P_KOMPASS_WIRKUNG] = get_Value(cb_7_3);
    ParameterSet[Set][P_UNTERSPANNUNG]   = sb_7_4->value();
    ParameterSet[Set][P_NOTGASZEIT]      = sb_7_5->value();
    ParameterSet[Set][P_NOTGAS]          = sb_7_6->value();

    // Seite 8
    ParameterSet[Set][P_ACHS_KOPPLUNG]  = get_Value(cb_8_1);
    ParameterSet[Set][P_ACHS_GKOPPLUNG] = get_Value(cb_8_2);

    // Seite 9
    ParameterSet[Set][P_LOOP_CONFIG] = 0;
    if (tb_9_6->text() == QString("1"))
        ParameterSet[Set][P_LOOP_CONFIG] = ParameterSet[Set][P_LOOP_CONFIG] | 0x01;
    if (tb_9_7->text() == QString("1"))
        ParameterSet[Set][P_LOOP_CONFIG] = ParameterSet[Set][P_LOOP_CONFIG] | 0x02;
    if (tb_9_8->text() == QString("1"))
        ParameterSet[Set][P_LOOP_CONFIG] = ParameterSet[Set][P_LOOP_CONFIG] | 0x04;
    if (tb_9_9->text() == QString("1"))
        ParameterSet[Set][P_LOOP_CONFIG] = ParameterSet[Set][P_LOOP_CONFIG] | 0x08;

    ParameterSet[Set][P_LOOP_GAS_LIMIT] = get_Value(cb_9_1);
    ParameterSet[Set][P_LOOP_THRESHOLD] = sb_9_2->value();
    ParameterSet[Set][P_WINKEL_NICK]    = sb_9_3->value();
    ParameterSet[Set][P_LOOP_HYSTERESE] = sb_9_4->value();
    ParameterSet[Set][P_WINKEL_ROLL]    = sb_9_5->value();

    // Seite 10
    ParameterSet[Set][P_USER_1] = get_Value(cb_10_1);
    ParameterSet[Set][P_USER_2] = get_Value(cb_10_2);
    ParameterSet[Set][P_USER_3] = get_Value(cb_10_3);
    ParameterSet[Set][P_USER_4] = get_Value(cb_10_4);
    ParameterSet[Set][P_USER_5] = get_Value(cb_10_5);
    ParameterSet[Set][P_USER_6] = get_Value(cb_10_6);
    ParameterSet[Set][P_USER_7] = get_Value(cb_10_7);
    ParameterSet[Set][P_USER_8] = get_Value(cb_10_8);

    // Seite 11
    ParameterSet[Set][P_J16_BITMASK] = sb_11_1->value();
    ParameterSet[Set][P_J16_TIMING]  = get_Value(cb_11_2);
    ParameterSet[Set][P_J17_BITMASK] = sb_11_3->value();
    ParameterSet[Set][P_J17_TIMING]  = get_Value(cb_11_4);

    // Seite 12
    ParameterSet[Set][P_NAV_GPS_MODE]  = get_Value(cb_12_1);
    ParameterSet[Set][P_NAV_GPS_GAIN]  = get_Value(cb_12_2);
    ParameterSet[Set][P_NAV_STICK_THRE] = sb_12_3->value();
    ParameterSet[Set][P_NAV_GPS_MIN] = sb_12_4->value();
    ParameterSet[Set][P_NAV_GPS_P]  = get_Value(cb_12_5);
    ParameterSet[Set][P_NAV_GPS_I]  = get_Value(cb_12_6);
    ParameterSet[Set][P_NAV_GPS_D]  = get_Value(cb_12_7);
    ParameterSet[Set][P_NAV_GPS_ACC]  = get_Value(cb_12_8);
}

void MKTool::show_ParameterSet(int Set)
{
    char Name[12];
    for (int a = 0; a < 12; a++)
    {
        Name[a] = ParameterSet[Set][P_NAME+a];
    }
    le_SetName->setText(QString(Name));

    // Seite 1
{
    cb_1_1->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x01);
    cb_1_2->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x02);
    cb_1_3->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x04);
    cb_1_4->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x08);
    cb_1_5->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x10);
    cb_1_6->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x20);
    cb_1_7->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x40);
    cb_1_8->setChecked(ParameterSet[Set][P_GLOBAL_CONF] & 0x80);
}
    // Seite 2
{
    sb_2_1->setValue(ParameterSet[Set][P_KANAL_NICK]);
    sb_2_2->setValue(ParameterSet[Set][P_KANAL_ROLL]);
    sb_2_3->setValue(ParameterSet[Set][P_KANAL_GAS]);
    sb_2_4->setValue(ParameterSet[Set][P_KANAL_GIER]);
    sb_2_5->setValue(ParameterSet[Set][P_KANAL_POTI1]);
    sb_2_6->setValue(ParameterSet[Set][P_KANAL_POTI2]);
    sb_2_7->setValue(ParameterSet[Set][P_KANAL_POTI3]);
    sb_2_8->setValue(ParameterSet[Set][P_KANAL_POTI4]);
}
    // Seite 3
{
    sb_3_1->setValue(ParameterSet[Set][P_STICK_P]);
    sb_3_2->setValue(ParameterSet[Set][P_STICK_D]);
    cb_3_3 = setCombo(cb_3_3, Set, P_GIER_P);
    cb_3_4 = setCombo(cb_3_4, Set, P_EXTERNAL);
}
    // Seite 4
{
    cb_4_1 = setCombo(cb_4_1, Set, P_MAXHOEHE);
    sb_4_2->setValue(ParameterSet[Set][P_MIN_GAS]);
    cb_4_3 = setCombo(cb_4_3, Set, P_HOEHE_P);
    cb_4_4 = setCombo(cb_4_4, Set, P_DRUCK_D);
    cb_4_5 = setCombo(cb_4_5, Set, P_HOEHE_ACC);
    sb_4_6->setValue(ParameterSet[Set][P_HOEHE_GAIN]);
}
    // Seite 5
{
    cb_5_1 = setCombo(cb_5_1, Set, P_GYRO_P);
    cb_5_2 = setCombo(cb_5_2, Set, P_GYRO_I);
    cb_5_3 = setCombo(cb_5_3, Set, P_DYNAMIC_STAB);
    sb_5_4->setValue(ParameterSet[Set][P_GYRO_ACC_FAKTOR]);
    sb_5_5->setValue(ParameterSet[Set][P_GYRO_ACC_ABGL]);
    cb_5_6 = setCombo(cb_5_6, Set, P_FAKTOR_I);
    sb_5_7->setValue(ParameterSet[Set][P_DRIFT_KOMP]);
}
    // Seite 6
{
    cb_6_1 = setCombo(cb_6_1, Set, P_SERVO_NICK_CONT);
    sb_6_2->setValue(ParameterSet[Set][P_SERVO_NICK_COMP]);
    sb_6_3->setValue(ParameterSet[Set][P_SERVO_NICK_MIN]);
    sb_6_4->setValue(ParameterSet[Set][P_SERVO_NICK_MAX]);
    sb_6_5->setValue(ParameterSet[Set][P_SERVO_NICK_REFR]);
    cb_6_6->setChecked(ParameterSet[Set][P_SERVO_NICK_COMPI]);
}
    // Seite 7
{
    sb_7_1->setValue(ParameterSet[Set][P_GAS_MAX]);
    sb_7_2->setValue(ParameterSet[Set][P_GAS_MIN]);
    cb_7_3 = setCombo(cb_7_3, Set, P_KOMPASS_WIRKUNG);
    sb_7_4->setValue(ParameterSet[Set][P_UNTERSPANNUNG]);
    sb_7_5->setValue(ParameterSet[Set][P_NOTGASZEIT]);
    sb_7_6->setValue(ParameterSet[Set][P_NOTGAS]);
}
    // Seite 8
{
    cb_8_1 = setCombo(cb_8_1, Set, P_ACHS_KOPPLUNG);
    cb_8_2 = setCombo(cb_8_2, Set, P_ACHS_GKOPPLUNG);
}
    // Seite 9
{
    if (ParameterSet[Set][P_LOOP_CONFIG] & 0x01)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-up-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_6->setIcon(Icon);
        tb_9_6->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-up.png")), QIcon::Normal, QIcon::Off);
        tb_9_6->setIcon(Icon);
        tb_9_6->setText("0");
    }

    if (ParameterSet[Set][P_LOOP_CONFIG] & 0x02)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-down-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_7->setIcon(Icon);
        tb_9_7->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-down.png")), QIcon::Normal, QIcon::Off);
        tb_9_7->setIcon(Icon);
        tb_9_7->setText("0");
    }

    if (ParameterSet[Set][P_LOOP_CONFIG] & 0x04)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-left-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_8->setIcon(Icon);
        tb_9_8->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-left.png")), QIcon::Normal, QIcon::Off);
        tb_9_8->setIcon(Icon);
        tb_9_8->setText("0");
    }

    if (ParameterSet[Set][P_LOOP_CONFIG] & 0x08)
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-right-double.png")), QIcon::Normal, QIcon::Off);
        tb_9_9->setIcon(Icon);
        tb_9_9->setText("1");
    }
    else
    {
        QIcon Icon;
        Icon.addPixmap(QPixmap(QString::fromUtf8(":/Arrows/Images/arrow-right.png")), QIcon::Normal, QIcon::Off);
        tb_9_9->setIcon(Icon);
        tb_9_9->setText("0");
    }

    cb_9_1 = setCombo(cb_9_1, Set, P_LOOP_GAS_LIMIT);
    sb_9_2->setValue(ParameterSet[Set][P_LOOP_THRESHOLD]);
    sb_9_3->setValue(ParameterSet[Set][P_WINKEL_NICK]);
    sb_9_4->setValue(ParameterSet[Set][P_LOOP_HYSTERESE]);
    sb_9_5->setValue(ParameterSet[Set][P_WINKEL_ROLL]);
}
    // Seite 10
{
    cb_10_1 = setCombo(cb_10_1, Set, P_USER_1);
    cb_10_2 = setCombo(cb_10_2, Set, P_USER_2);
    cb_10_3 = setCombo(cb_10_3, Set, P_USER_3);
    cb_10_4 = setCombo(cb_10_4, Set, P_USER_4);
    cb_10_5 = setCombo(cb_10_5, Set, P_USER_5);
    cb_10_6 = setCombo(cb_10_6, Set, P_USER_6);
    cb_10_7 = setCombo(cb_10_7, Set, P_USER_7);
    cb_10_8 = setCombo(cb_10_8, Set, P_USER_8);
}
    // Seite 11
{
    sb_11_1->setValue(ParameterSet[Set][P_J16_BITMASK]);
    cb_11_2 = setCombo(cb_11_2, Set, P_J16_TIMING);
    sb_11_3->setValue(ParameterSet[Set][P_J17_BITMASK]);
    cb_11_4 = setCombo(cb_11_4, Set, P_J17_TIMING);
}
    // Seite 12
{
    cb_12_1 = setCombo(cb_12_1, Set, P_NAV_GPS_MODE);
    cb_12_2 = setCombo(cb_12_2, Set, P_NAV_GPS_GAIN);
    sb_12_3->setValue(ParameterSet[Set][P_NAV_STICK_THRE]);
    sb_12_4->setValue(ParameterSet[Set][P_NAV_GPS_MIN]);
    cb_12_5 = setCombo(cb_12_5, Set, P_NAV_GPS_P);
    cb_12_6 = setCombo(cb_12_6, Set, P_NAV_GPS_I);
    cb_12_7 = setCombo(cb_12_7, Set, P_NAV_GPS_D);
    cb_12_8 = setCombo(cb_12_8, Set, P_NAV_GPS_ACC);

}
}

void MKTool::new_RXData(QString RX_Data)
{
    char *RX = RX_Data.toLatin1().data();

    if (LastSend.length() > 2)
    {
    }

    switch(RX[2])
    {
        case 'D' :
                    le_In->setText(RX_Data);
                    Decode64(RX_Data);
                    for (int i=0; i<32; i++)
                    {
                        AnalogData[i] = DecodeData[i+1];
                    }
                    show_DebugData();
                break;
        case 'L' :
        case 'M' :
        case 'N' :
        case 'O' :
        case 'P' :
                    timer->stop();

                    Decode64(RX_Data, true);
                    memcpy(ParameterSet[sb_Set->value()],DecodeData, sizeof(DecodeData));
                    show_ParameterSet(sb_Set->value());
                break;
        case 'V' :
                    timer->stop();

                    le_In->setText(RX_Data);

                    Decode64(RX_Data, true);
                    QString Version = QString("%1").arg(DecodeData[0]) + "." + QString("%1").arg(DecodeData[1]);
                    setWindowTitle(QA_NAME + " v" + QA_VERSION + " - FC Version: " + Version);

                    if (DecodeData[2] != FC_VERSION)
                    {
                        pb_Read->setDisabled(true);
                        pb_Write->setDisabled(true);

                        QMessageBox::warning(this, QA_NAME,
                           QA_NAME + " und Flight-Control inkompatibel.\nParameterbearbeitung nicht moeglich.", QMessageBox::Ok);
                    }
                    else
                    {
                        TX_Data[0] = 1;
                        TX_Data[1] = 0;
                        send_Data('q', TX_Data, 1);
                    }
                break;
    }
}

QString MKTool::add_CRC(QString TXString)
{
    unsigned int tmpCRC = 0;

    char *TXBuff;
    char CRC[2];

    TXBuff = TXString.toLatin1().data();

    for(int i = 0; i < TXString.length(); i++)
    {
        tmpCRC += TXBuff[i];
    }

    tmpCRC %= 4096;

    CRC[0] = '=' + tmpCRC / 64;
    CRC[1] = '=' + tmpCRC % 64;
    CRC[2] = '\0';

    QString Return = TXString + QString(CRC);

    return Return;
}

bool MKTool::check_CRC(QString RXString)
{
    int CRC = 0;
    char *RX;

    int Length = RXString.length();

    RX = RXString.toLatin1().data();

    for(int i=0; i < Length-2; i++)
    {
            CRC+=RX[i];
    }

    CRC -= RX[1];

    CRC = CRC % 4096;

    if(RX[Length - 2] != ('=' + (CRC / 64)))
    {
        return false;
    }

    if(RX[Length - 1] != ('=' + CRC % 64))
    {
        return false;
    }

    return true;
}

void MKTool::slot_Send()
{

}

void MKTool::slot_Timer()
{
    QByteArray Temp(LastSend.toUtf8());
    serialPort->sendData(Temp);
    qDebug("Re-Send");
}

void MKTool::slot_OpenPort()
{
    if (serialPort->isOpen())
    {
        timer->stop();
        serialPort->close();
        pb_Open->setText("Verbinden");
    }
    else
    {
        serialPort->setPort(le_Port->text()); //Port

        serialPort->setBaudRate(BAUD57600); //BaudRate
        serialPort->setDataBits(DATA_8); //DataBits
        serialPort->setParity(PAR_NONE); //Parity
        serialPort->setStopBits(STOP_1); //StopBits
        serialPort->setFlowControl(FLOW_OFF); //FlowControl

        serialPort->setTimeout(0, 10);
        serialPort->enableSending();
        serialPort->enableReceiving();

        serialPort->open();
        serialPort->receiveData();

        send_Data('v', TX_Data, 0);

        pb_Open->setText("Trennen");
    }
}

void MKTool::slot_newDataReceived(const QByteArray &dataReceived)
{
    allDataReceived += dataReceived;

    te_RX->moveCursor(QTextCursor::End,  QTextCursor::MoveAnchor);

    QTextStream in(dataReceived);

    QString decodedStr = in.readAll();

    const char *RXt;
    RXt = dataReceived.data();
    int a = 0;

    while (RXt[a] != '\0')
    {
        if (RXt[a] == '\r')
        {
            while ((RXS.length() > 1) && (RXS.at(1) == '#'))
            {
                RXS.remove(0,1);
            }
            if (check_CRC(RXS))
            {
                te_RX->insertPlainText(RXS + '\r');
                new_RXData(RXS);
            }
            else
            {
//                te_RX->insertPlainText(RXS + '\r');
            }

            RXS = QString("");
        }
        else
        {
            RXS = RXS + QString(RXt[a]);
        }
            a++;
    }
}

void MKTool::slot_GetParameter()
{
    TX_Data[0] = sb_Set->value();
    TX_Data[1] = 0;
    send_Data('q', TX_Data, 1);
}

void MKTool::slot_SetParameter()
{
    store_ParameterSet(sb_Set->value());

    for (int a = 0; a < 150; a++)
    {
        TX_Data[a] = ParameterSet[sb_Set->value()][a];
    }

    store_ParameterSet(sb_Set->value());

    char Set = 'k' + sb_Set->value();
    send_Data(Set, TX_Data, 83);
    timer->stop();
}

void MKTool::slot_Quit()
{
    if (serialPort->isOpen())
    {
        serialPort->close();
    }

    write_Settings();
    close();
}

MKTool::~MKTool()
{
    write_Settings();
}

